import { DateSelectArg } from "@fullcalendar/core";
import frLocale from "@fullcalendar/core/locales/fr";
import dayGridPlugin from "@fullcalendar/daygrid";
import interactionPlugin from "@fullcalendar/interaction";
import listPlugin from "@fullcalendar/list";
import FullCalendar from "@fullcalendar/react";
import {
  Alert,
  Checkbox,
  FormControl,
  FormControlLabel,
  FormGroup,
  FormLabel,
} from "@mui/material";
import React from "react";
import { FamilyApi, FamilyUser } from "../../../api/FamilyApi";
import {
  AccommodationsReservationsApi,
  AccommodationsReservationsList,
} from "../../../api/accommodations/AccommodationsReservationsApi";
import { useAlert } from "../../../hooks/context_providers/AlertDialogProvider";
import { useLoadingMessage } from "../../../hooks/context_providers/LoadingMessageProvider";
import { useSnackbar } from "../../../hooks/context_providers/SnackbarProvider";
import { useUpdateAccommodationReservation } from "../../../hooks/context_providers/accommodations/UpdateReservationDialogProvider";
import { fmtUnixDateFullCalendar } from "../../../utils/time_utils";
import { AsyncWidget } from "../../../widgets/AsyncWidget";
import { useFamily } from "../../../widgets/BaseFamilyRoute";
import { FamilyPageTitle } from "../../../widgets/FamilyPageTitle";
import { useAccommodations } from "../../../widgets/accommodations/BaseAccommodationsRoute";

export function AccommodationsReservationsRoute(): React.ReactElement {
  const snackbar = useSnackbar();
  const alert = useAlert();
  const loadingMessage = useLoadingMessage();

  const loadKey = React.useRef(1);

  const family = useFamily();
  const accommodations = useAccommodations();
  const updateReservation = useUpdateAccommodationReservation();

  const [reservations, setReservations] = React.useState<
    AccommodationsReservationsList | undefined
  >();
  const [users, setUsers] = React.useState<FamilyUser[] | null>(null);

  const [showValidated, setShowValidated] = React.useState(true);
  const [showRejected, setShowRejected] = React.useState(true);
  const [showPending, setShowPending] = React.useState(true);

  const [hiddenPeople, setHiddenPeople] = React.useState<Set<number>>(
    new Set()
  );
  const [hiddenAccommodations, setHiddenAccommodations] = React.useState<
    Set<number>
  >(new Set());

  const load = async () => {
    setReservations(
      await AccommodationsReservationsApi.FullListOfFamily(family.family)
    );
    setUsers(await FamilyApi.GetUsersList(family.family.family_id));
  };

  const reload = async () => {
    loadKey.current += 1;
    setUsers(null);
  };

  const visibleReservations = React.useMemo(() => {
    return reservations?.filter((r) => {
      if (!showValidated && r.validated === true) return false;
      if (!showPending && r.validated === null) return false;
      if (!showRejected && r.validated === false) return false;
      if (hiddenPeople.has(r.user_id)) return false;
      if (hiddenAccommodations.has(r.accommodation_id)) return false;
      return true;
    });
  }, [
    showValidated,
    showRejected,
    showPending,
    hiddenPeople,
    hiddenAccommodations,
    reservations,
  ]);

  const onSelect = async (d: DateSelectArg) => {
    try {
      const resa = await updateReservation(
        {
          accommodation_id: -1,
          start: Math.floor(d.start.getTime() / 1000),
          end: Math.floor(d.end.getTime() / 1000),
        },
        true
      );

      if (!resa) return;

      loadingMessage.show("Création de la réservation en cours...");

      await AccommodationsReservationsApi.Create(family.family, resa);

      reload();
      snackbar("La réservation a été créée avec succès !");
    } catch (e) {
      console.error("Failed to create a reservation!", e);
      alert("Échec de la création de la réservation!");
    } finally {
      loadingMessage.hide();
    }
  };

  return (
    <>
      <FamilyPageTitle title="Réservations" />
      <AsyncWidget
        loadKey={loadKey.current}
        load={load}
        errMsg="Echec du chargement de la liste des réservations !"
        build={() => (
          <div style={{ display: "flex", flexDirection: "row" }}>
            <div style={{ flex: 1, maxWidth: "250px", marginRight: "20px" }}>
              <Alert severity="info">
                Cliquez sur le calendrier pour créer une réservation.
              </Alert>

              {/* Invitation status */}
              <FormControl
                sx={{ m: 3 }}
                component="fieldset"
                variant="standard"
              >
                <FormLabel component="legend">Status</FormLabel>
                <FormGroup>
                  <FormControlLabel
                    control={
                      <Checkbox
                        checked={showValidated}
                        onChange={(_ev, v) => setShowValidated(v)}
                        color="success"
                      />
                    }
                    label="Validées"
                  />
                  <FormControlLabel
                    control={
                      <Checkbox
                        checked={showRejected}
                        onChange={(_ev, v) => setShowRejected(v)}
                        color="error"
                      />
                    }
                    label="Rejetées"
                  />
                  <FormControlLabel
                    control={
                      <Checkbox
                        checked={showPending}
                        onChange={(_ev, v) => setShowPending(v)}
                        color="info"
                      />
                    }
                    label="En attente de validation"
                  />
                </FormGroup>
              </FormControl>

              {/* Accommodations */}
              <FormControl
                sx={{ m: 3 }}
                component="fieldset"
                variant="standard"
              >
                <FormLabel component="legend">Logements</FormLabel>
                <FormGroup>
                  {accommodations.accommodations.fullList.map((a) => (
                    <FormControlLabel
                      key={a.id}
                      control={
                        <Checkbox
                          sx={{
                            color: "#" + a.color,
                            "&.Mui-checked": {
                              color: "#" + a.color,
                            },
                          }}
                          checked={!hiddenAccommodations.has(a.id)}
                          onChange={(_ev, v) => {
                            if (v) hiddenAccommodations.delete(a.id);
                            else hiddenAccommodations.add(a.id);
                            setHiddenAccommodations(
                              new Set(hiddenAccommodations)
                            );
                          }}
                        />
                      }
                      label={a.name}
                    />
                  ))}
                </FormGroup>
              </FormControl>

              {/* People */}
              <FormControl
                sx={{ m: 3 }}
                component="fieldset"
                variant="standard"
              >
                <FormLabel component="legend">Personnes</FormLabel>
                <FormGroup>
                  {users?.map((u) => (
                    <FormControlLabel
                      key={u.user_id}
                      control={
                        <Checkbox
                          checked={!hiddenPeople.has(u.user_id)}
                          onChange={(_ev, v) => {
                            if (v) hiddenPeople.delete(u.user_id);
                            else hiddenPeople.add(u.user_id);
                            setHiddenPeople(new Set(hiddenPeople));
                          }}
                        />
                      }
                      label={u.user_name}
                    />
                  ))}
                </FormGroup>
              </FormControl>
            </div>

            {/* The calendar */}
            <div style={{ flex: 5 }}>
              <FullCalendar
                editable={true}
                selectable={true}
                plugins={[dayGridPlugin, listPlugin, interactionPlugin]}
                initialView="dayGridMonth"
                height="700px"
                locale={frLocale}
                headerToolbar={{
                  left: "prev,next today",
                  center: "title",
                  right: "dayGridMonth,dayGridWeek,dayGridDay,listWeek",
                }}
                select={onSelect}
                events={visibleReservations?.map((r) => {
                  const a = accommodations.accommodations.get(
                    r.accommodation_id
                  )!;
                  const u = users?.find((u) => u.user_id === r.user_id);
                  return {
                    title: `${u?.user_name} - ${a.name}`,
                    start: fmtUnixDateFullCalendar(r.reservation_start),
                    end: fmtUnixDateFullCalendar(r.reservation_end),
                    color: a.color ? "#" + a.color : undefined,
                    borderColor:
                      r.validated === true
                        ? "green"
                        : r.validated === false
                        ? "red dotted"
                        : "grey dotted",
                  };
                })}
              />
            </div>
          </div>
        )}
      />
    </>
  );
}
